﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using FlowBasis.Json;

namespace FlowBasisUnitTests.Json
{
    public class JsonSerializationServiceTests
    {
        [Test]
        public void Test_Stringify()
        {
            Func<IJObjectRootMapper> rootMapperFactory = () => new JObjectRootMapper();
            var jsonSerializer = new JsonSerializationService(rootMapperFactory);

            Assert.AreEqual("null", jsonSerializer.Stringify(null));
            Assert.AreEqual("4", jsonSerializer.Stringify(4));
            Assert.AreEqual("\"Hello\"", jsonSerializer.Stringify("Hello"));

            Assert.AreEqual("{\"someNumber\":42}", jsonSerializer.Stringify(new TestObject { SomeNumber = 42 }));
        }

        [Test]
        public void Test_Parse()
        {
            Func<IJObjectRootMapper> rootMapperFactory = () => new JObjectRootMapper();
            var jsonSerializer = new JsonSerializationService(rootMapperFactory);

            Assert.IsNull(jsonSerializer.Parse("null"));
            Assert.AreEqual(4, jsonSerializer.Parse("4"));
            Assert.AreEqual("Hello", jsonSerializer.Parse("\"Hello\""));

            JObject resultJObject = (JObject)jsonSerializer.Parse("{\"someNumber\":42}");
            Assert.AreEqual(42, resultJObject["someNumber"]);

            TestObject resultTestObject = (TestObject)jsonSerializer.Parse("{\"someNumber\":43}", typeof(TestObject));
            Assert.AreEqual(43, resultTestObject.SomeNumber);

            resultTestObject = jsonSerializer.Parse<TestObject>("{\"someNumber\":44}");
            Assert.AreEqual(44, resultTestObject.SomeNumber);
        }


        public class TestObject
        {
            public int SomeNumber { get; set; }
        }
    }
}
