﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using FlowBasis.Json;

namespace FlowBasisUnitTests.Json
{
    public class JsonTypeAttributeTests
    {
        [Test]
        public void Test_MapperType()
        {
            var jsonSerializer = new JsonSerializationService(() => new JObjectRootMapper());

            // TestObject should get converted into reversed case string when serialized with custom mapper.
            string json = jsonSerializer.Stringify(new TestObject() { SomeStr = "Abc" });
            Assert.AreEqual("\"aBC\"", json);

            // And it should be deserialized back into TestObject from a string.
            var result = jsonSerializer.Parse<TestObject>(json);
            Assert.AreEqual("Abc", result.SomeStr);
        }


        [JsonType(MapperType = typeof(TestObjectMapper))]
        public class TestObject
        {
            public string SomeStr { get; set; }
        }

        public class TestObjectMapper : IJObjectMapper
        {
            public object ToJObject(object instance, IJObjectRootMapper rootMapper)
            {
                TestObject testObject = (TestObject)instance;

                StringBuilder sb = new StringBuilder();
                foreach (char ch in testObject.SomeStr)
                {
                    if (Char.IsLower(ch))
                    {
                        sb.Append(Char.ToUpper(ch));
                    }
                    else
                    {
                        sb.Append(Char.ToLower(ch));
                    }
                }

                return sb.ToString();
            }

            public object FromJObject(object jObject, Type targetType, IJObjectRootMapper rootMapper)
            {
                string str = (string)jObject;

                StringBuilder sb = new StringBuilder();
                foreach (char ch in str)
                {
                    if (Char.IsLower(ch))
                    {
                        sb.Append(Char.ToUpper(ch));
                    }
                    else
                    {
                        sb.Append(Char.ToLower(ch));
                    }
                }

                return new TestObject() { SomeStr = sb.ToString() };
            }
        }
    }
}
