﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using FlowBasis.Security.Passwords;

namespace FlowBasisUnitTests.Security.Passwords
{
    public class Sha512PasswordValidationServiceTests
    {
        [Test]
        public void Should_Generate_Random_Salt()
        {
            var passwordService = new Sha512PasswordValidationService();

            byte[] salt1 = passwordService.GenerateSalt(64);
            byte[] salt2 = passwordService.GenerateSalt(64);

            Assert.AreEqual(64, salt1.Length);
            Assert.AreEqual(64, salt2.Length);

            Assert.AreElementsNotEqual(salt1, salt2);
        }

        [Test]
        public void Should_Generate_Hash_And_Validate_Password()
        {
            var passwordService = new Sha512PasswordValidationService();

            byte[] saltCorrect = passwordService.GenerateSalt(64);
            byte[] saltWrong = passwordService.GenerateSalt(64);

            byte[] hash = passwordService.ComputeHash("hello", saltCorrect);

            bool result = passwordService.ValidatePassword("hello", saltCorrect, hash);
            Assert.IsTrue(result);

            result = passwordService.ValidatePassword("hello", saltWrong, hash);
            Assert.IsFalse(result);

            result = passwordService.ValidatePassword("helloWrong", saltCorrect, hash);
            Assert.IsFalse(result);
        }
    }
}
